/**
 * $Id: fp_return.h,v 1.11 2002/10/29 18:29:09 plasmahh Exp $
 * $Revision: 1.11 $
 * $Author: plasmahh $
 * $Date: 2002/10/29 18:29:09 $
 */
#ifndef __FP_RETURN_H
#define __FP_RETURN_H

#ifndef __FP_H
#include "fp.h"
#endif 

/**
 * @brief Generic Fingerprint
 *
 * This class will be a generic fingerprint. It is a dllist of
 * checksums from all the different returns in answers from
 * (smtp) server
 * @todo Create <,> and == operators to sort those fps.... perhaps
 * better do this in base class, since it should read error, and it
 * is defined there...
 */
class fp_return : public fp, public dllist<unsigned long> 
{
public:
    /**
     * constructs this fingerprint
     * @param Version stirng this fingerprint is of
     */
    fp_return( const mstring& ver ) : fp(ver,FP_Return){}

    /**
     * Copy constructor
     */
    fp_return( const fp_return& ofp) : fp(ofp.version,FP_Return){ *this = ofp; }

/**
 * Desctructor, will do nothing, the base classes will destroy data
 */
virtual ~fp_return(){};
/**
 * adds a return to this fingerprint. The fingerprint will
 * automatically determine what information will be used
 * @param string with the return to actual command
 */
virtual void add( const mstring& ret);

/**
 * adds a value to the fingerprint. Most likely used to build
 * up a saved fingerprint instead of creating a new one from 
 * server returns.
 */
virtual void add( unsigned long fp) { dllist<unsigned long>::add(fp); }

/**
 * This is the nice part. We get another fingerprint end compute
 * the difference to us and store it in error. You can do this with a 
 * list of fps and set all those errors
 */
virtual unsigned long gen_error( fp_return & ofp);
friend ostream& operator << (ostream &o, fp_return &v);
friend bool operator < ( const fp_return&, const fp_return&);
friend bool operator > ( const fp_return&, const fp_return&);
};

#else
#warning Multiple inclusion of fp_return.h check this,shouldnt be possible
#endif
/**
 *$Log: fp_return.h,v $
 *Revision 1.11  2002/10/29 18:29:09  plasmahh
 *fix no-compile for gcc 3.2
 *
 *Revision 1.10  2002/10/21 08:38:28  plasmahh
 *begin cleanup and structure changes for smtpmap 0.8-stable
 *
 *Revision 1.9  2002/06/20 20:59:36  plasmahh
 *internal documentation changes
 *
 *Revision 1.8  2002/06/17 21:32:36  plasmahh
 *Fixed, so it compiles on g++3.1 and later
 *
 *Revision 1.7  2002/06/09 21:22:39  plasmahh
 *lots of changes
 *some slight fingerprint changes, also in constructor
 *scanner changes and fixes in constructor
 *read fixes
 *test programs fixes, no more segfaults
 *new TODO
 *
 *Revision 1.6  2002/05/23 22:37:55  plasmahh
 *fixed small things in scanner
 *split fps into .cpp files, because of linker errors in operators
 *
 *Revision 1.5  2002/05/23 19:50:04  plasmahh
 *refined include structure.
 *implemented and fixed some parts, still not working
 *BUG !! FIX THIS !!! : multiple definition of some operators, we will never compile !!
 *
 *Revision 1.4  2002/05/14 16:31:59  plasmahh
 *implementing new scanner structure, that will be   e xtendible
 *
 *Revision 1.3  2002/05/13 13:56:46  plasmahh
 **** empty log message ***
 *
 *Revision 1.2  2002/05/06 21:27:44  plasmahh
 *redesigned the classes a bit...
 *begin  implementing main & static
 *
 *Revision 1.1  2002/05/05 20:25:29  plasmahh
 *Added new fingerprint classes... will delete older soon
 *
 */
